from mext.dispatch import Adapter

__all__ = [
    'as_xml', 'as_xmlnode',
    'EscapedXML', 'XmlEntity', 'NOTAG',
    'XmlDocument', 'XmlElement',
    'xmlelem',
]


def escape_text(text):
    return text.replace("&", "&amp;").replace(">", "&gt;").replace("<", "&lt;")

def escape_attribute(text):
    text = escape_text(text)
    text = text.replace('\n', '&#10;').replace('\r', '&#13;').replace('\t', '&#9;')
    if '"' in text:
        if "'" in text:
            text = '"%s"' % text.replace('"', "&quot;")
        else:
            text = "'%s'" % text
    else:
        text = '"%s"' % text
    return text



as_xmlnode = Adapter('__xmlnode__')

as_xml = Adapter('__xml__')
as_xml[object] = lambda ob: as_xml(as_xmlnode(ob))
as_xml[None] = lambda none: u''
as_xml[str] = lambda s: escape_text(unicode(s))
as_xml[unicode] = escape_text
as_xml[list] = as_xml[tuple] = lambda ob: u''.join(map(as_xml, ob))


class EscapedXML(unicode):
    __slots__ = ()
    def __xml__(self):
        return self

def XmlEntity(code):
    if isinstance(code, int):
        code = '#x%X' % code
    return EscapedXML('&%s;' % code)






class XmlElement(object):
    __slots__ = ['tag', 'attrs', 'body', '_cache']
    def __init__(self, __tag, __body=None, **kw):
        self.tag = __tag
        self.body = [__body] if __body is not None else []
        self.attrs = dict((key.rstrip('_').replace('_', '-'), value)
                                    for key, value in kw.iteritems())
        self._cache = None

    def __iadd__(self, item):
        self._cache = None
        self.body.append(item)
        return self

    def __repr__(self):
        return (self.__class__.__name__
                + ('(%s)' % ', '.join(   [repr(self.tag)]
                                        + ['%s=%r' % attr for attr in self.attrs.iteritems()]))
                + ('[...]' if self.body else ''))

    def __getitem__(self, body):
        if self.body:
            raise ValueError("%s elem already has a body: %r" % (self.tag, self.body))
        elem = XmlElement(self.tag, body)
        elem.attrs = self.attrs.copy()
        return elem

    def __xml__(self):
        if self._cache is not None:
            return self._cache
        if self.attrs:
            attrs = u''.join(u' %s=%s' % (key, escape_attribute(unicode(value)))
                            for key, value in self.attrs.iteritems() if value is not None)
        else:
            attrs = u''
        if not self.body:
            result = u'<%s%s/>' % (self.tag, attrs)
        else:
            result = u'<%s%s>%s</%s>' % (self.tag, attrs, as_xml(self.body), self.tag)
        self._cache = result
        return result





class XmlElementGen(object):
    __slots__ = ['tag', 'empty']
    elem_cls = XmlElement
    def __init__(self, tag):
        self.tag = tag
        self.empty = as_xml(self.elem_cls(self.tag))

    def __xml__(self):
        return self.empty

    #def __iadd__(self, item):
    #    return self[item]

    def __call__(self, **attrs):
        return self.elem_cls(self.tag, **attrs)

    def __getitem__(self, body):
        return self.elem_cls(self.tag, body)
    __iadd__ = __getitem__


class XmlDocument(object):
    mime_type = 'application/xml'
    _xml_decl_template = u'<?xml version="1.0" encoding="%s" ?>\n'
    _xml_decl_unicode = u'<?xml version="1.0" ?>\n'
    def __init__(self, content, encoding='UTF-8'):
        self.content = as_xml(content)
        self.encoding = encoding

    def __unicode__(self):
        return self._xml_decl_unicode + self.content

    def __str__(self):
        return self.encode(self.encoding)

    def __mimedata__(self):
        from mext.mime import MimeTypedData
        return MimeTypedData(str(self), self.mime_type, charset=self.encoding)

    def __webob__(self):
        from webob import Response
        return Response(str(self), content_type=self.mime_type, charset=self.encoding)

    def encode(self, encoding):
        return ((self._xml_decl_template % encoding) + self.content).encode(encoding)





class _AttrGen(object):
    def __getattr__(self, name):
        return XmlElementGen(name)

xmlelem = _AttrGen()





class _NoTag(list):
    def __iadd__(self, item):
        self.append(item)
        return self

    def __repr__(self):
        return '_NoTag(%s)' % super(_NoTag, self).__repr__()


class NOTAG(object):
    def __call__(self):
        return _NoTag()

    @staticmethod
    def __xml__():
        return u''

    @staticmethod
    def __getitem__(item):
        return _NoTag([item])
    __iadd__ = __getitem__

NOTAG = NOTAG()









